unit fMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ActnList, XPMan, ImgList, ComCtrls, Menus, ToolWin,
  ExtCtrls;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    ToolBar1: TToolBar;
    File1: TMenuItem;
    Edit1: TMenuItem;
    VIew1: TMenuItem;
    Help1: TMenuItem;
    OpenButton: TToolButton;
    SaveButton: TToolButton;
    ImageList: TImageList;
    ActionList: TActionList;
    XPManifest: TXPManifest;
    OpenAction: TAction;
    SaveAction: TAction;
    Open1: TMenuItem;
    Save1: TMenuItem;
    Memo: TMemo;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    AboutAction: TAction;
    About1: TMenuItem;
    BottomPanel: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure AboutActionExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure MemoChange(Sender: TObject);
    procedure OpenActionExecute(Sender: TObject);
    procedure SaveActionExecute(Sender: TObject);
  private
    { Private declarations }
    FChangedSinceSave: Boolean;
    function SaveHandled: Boolean;
    function SaveContents: Boolean;
    procedure HideAppFormTaskBarButton;
    procedure SetupGlass;
  protected 
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WMSyscommand(var Message: TWmSysCommand); message WM_SYSCOMMAND;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses fAbout, uVistaFuncs;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  SetVistaFonts(Self);
  HideAppFormTaskBarButton;
  SetupGlass;
end;

procedure TMainForm.SetupGlass;
begin
  if CompositingEnabled then
  begin
    BottomPanel.Color := clBlack;
    ExtendGlass(Self.Handle, Rect(0, 0, 0, 24));
  end else
  begin
    BottomPanel.Color := Color;
  end;
end;

procedure TMainForm.HideAppFormTaskBarButton;
begin
  ShowWindow(Application.Handle, SW_HIDE);
  SetWindowLong(Application.Handle, GWL_EXSTYLE,
    GetWindowLong(Application.Handle, GWL_EXSTYLE) and not WS_EX_APPWINDOW
    or WS_EX_TOOLWINDOW);
  ShowWindow(Application.Handle, SW_SHOW);
end;

procedure TMainForm.AboutActionExecute(Sender: TObject);
begin
  AboutForm := TAboutForm.Create(Self);
  try
    AboutForm.PopupParent := Self;
    AboutForm.ShowModal;
  finally
    FreeAndNil(AboutForm);
  end;
end;

procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := Params.ExStyle and not WS_EX_TOOLWINDOW or
    WS_EX_APPWINDOW;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := SaveHandled;
end;

procedure TMainForm.MemoChange(Sender: TObject);
begin                                            
  FChangedSinceSave := True;
end;

function TMainForm.SaveHandled: Boolean;
begin
  Result := not FChangedSinceSave;
  if not Result then
  begin
    case TaskDialog(Self.Handle, 'Save Changes',
      'The contents of your document have changed',
      'Would you like to save your changes?  Click No only if you ' + #13 + #10 +
      'wish to abandon any changes to your document.',
      TD_ICON_QUESTION, TD_BUTTON_YES + TD_BUTTON_NO + TD_BUTTON_CANCEL) of
      TD_RESULT_YES: Result := SaveContents;
      TD_RESULT_NO: Result := True;
    end;
  end;
end;

procedure TMainForm.WMSyscommand(var Message: TWmSysCommand);
begin
  case (Message.CmdType and $FFF0) of
    SC_MINIMIZE:
    begin
      ShowWindow(Handle, SW_MINIMIZE);
      Message.Result := 0;
    end;
    SC_RESTORE:
    begin
      ShowWindow(Handle, SW_RESTORE);
      Message.Result := 0;
    end;
  else
    inherited;
  end;
end;

procedure TMainForm.OpenActionExecute(Sender: TObject);
begin
  if SaveHandled and OpenDialog.Execute then
  begin
    Memo.Lines.LoadFromFile(OpenDialog.FileName);
    FChangedSinceSave := True;
  end;
end;

function TMainForm.SaveContents: Boolean;
begin
  Result := False;
  if SaveDialog.Execute then
  begin
    Memo.Lines.SaveToFile(SaveDialog.FileName);
    FChangedSinceSave := True;
    Result := True;
  end;
end;

procedure TMainForm.SaveActionExecute(Sender: TObject);
begin
  SaveContents;
end;

end.
